/**  Little program for color transfer
*
* @author
*   Gabriel Zachmann, Sep 2006
*
* @bug
*   I'm sure there is still some bugs left ....
**/

// Bearbeitet von:
//  Markus Kramer   (342195)
//  Marcus Petersen (341572)
//  Daniel Mohr

#include <math.h>

#include "Picturearea.h"

PictureArea::PictureArea(QWidget *parent)
: QWidget(parent)
, m_originalImage(0)
, m_eqalizedImage(0)
, m_colorSpace(RGB)
{
	m_Histo = new int *[3];
	for ( int c = 0; c < 3; c++ )
		m_Histo[c] = new int[NUM_BINS];

	m_equalize_chan[0] = m_equalize_chan[1] = m_equalize_chan[2] = false;
}

PictureArea::~PictureArea()
{
	delete m_originalImage;
	delete m_eqalizedImage;
	
	for ( int c = 0; c < 3; c++ )
		delete [] m_Histo[c];
	delete [] m_Histo;

}

void PictureArea::paintEvent(QPaintEvent *)
{
	if ( 0 != m_eqalizedImage )
	{
		QPainter painter(this);
		painter.drawImage(this->rect(), *m_eqalizedImage);
	}
}


void PictureArea::slotSetImage(const QImage &image)
{
	delete m_originalImage;
	delete m_eqalizedImage;
	m_originalImage = new QImage(image);
	m_eqalizedImage = new QImage(image.width(),image.height(),image.format());
	equalizeImage(*m_originalImage,*m_eqalizedImage);
	update();
}

void PictureArea::mousePressEvent( QMouseEvent *e )
{
	emit changeHistogram(m_Histo,NUM_BINS);
}

void PictureArea::resetHistogram(int channel)
{
	for ( int i = 0; i < 256; i++)
		m_Histo[channel][i] = 0;
}

void PictureArea::setHistogram2Identity(int channel)
{
	for ( int i = 0; i < 256; i++)
		m_Histo[channel][i] = i;
}
// compute histogram for channel i: 
// -> m_Histo[i] <-
void PictureArea::computeHistogram(const QImage &imageIn)
{
	// ** TODO **
	// ....
}

void PictureArea::equalizeImage(const QImage &imageIn, QImage &imageOut)
{
	// Tips:
	// Pixel aus QImage lesen: QRgb color = image.pixel(x,y)
	// farbkanal extrahieren: int red = qRed(color); int green = qGreen(color); int blue = qBlue(color);
	// Pixel in QImage setzen: imageOut.setPixel( x, y, QColor::fromRgb(r,g,b).rgb() )

	// ** TODO **
	// Equalize channel i iff m_equalize_chan[i] == true
	imageOut = imageIn; // ** replace this **

	// recompute histogram of final image (visualization purposes on bottom right window)
	computeHistogram(imageOut);
}

void PictureArea::RGB2HSV(float r, float g, float b, float *h, float *s, float *v)
{
	/* this Code is copied from Computer Graphics, Principles an Practice, Foley et al */
	/* Given: r,g,b, each in [0,1] */
	/* Desired: h in [0,360), s an v in [0,1], except if s=0, then h = UNDEFINED */
	float max = std::max(std::max(r,g),b);
	float min = std::min(std::min(r,g),b);
	*v = max;
	*s = (max != 0.0f) ? ((max-min)/max):0.0f;
	if(*s == 0.0f) {
		*h = max*360.0f; // Hue is undefined here, so we can assigh what we want... we chose to assign max
	} else {
		float delta = max-min;
		if(r==max)
			*h = (g-b)/delta;
		else if (g == max)
			*h = 2.0f+(b-r)/delta;
		else if (b==max)
			*h = 4.0f + (r-g)/delta;
		*h *= 60.0f;
		if(*h < 0.0f)
			*h += 360.0f;
	}
	//*s*=*v;
}

void PictureArea::HSV2RGB(float h, float s, float v, float *r, float *g, float *b)
{
	/* this Code is copied from Computer Graphics, Principles an Practice, Foley et al */
  /* Given: h in [0,360] or UNDEFINED, s and v in [0,1] */
	/* Desired: r,g,b, each in [0,1] */
	if(s==0) 
	{
		if(h==-1.0) 
		{
			*r = v;
			*g = v;
			*b = v;
		} else 
		{
			fprintf(stderr,"hsv -> rgb conversion error (s==0 and h > 0)\n");
			exit(1);
		}
	} 
	else 
	{
		float f,p,q,t;
		int i;

		if (h==360.0)
			h = 0.0;
		h /= 60.0;
		i = static_cast<int>(floor(h));
		f = h-i;
		p = v*(1.0f-s);
		q = v*(1.0f-(s*f));
		t = v*(1.0f-(s*(1.0f-f)));
		switch(i) {
				case 0: *r = v; *g = t; *b = p; break;
				case 1: *r = q; *g = v; *b = p; break;
				case 2: *r = p; *g = v; *b = t; break;
				case 3: *r = p; *g = q; *b = v; break;
				case 4: *r = t; *g = p; *b = v; break;
				case 5: *r = v; *g = p; *b = q; break;
		}
	}
}